#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <string.h>
#include <signal.h>
#include <stdlib.h>
#include <linux/usbdevice_fs.h>

#include "lirc-min.h"

#define BUFFER_SIZE 100
#define DEFAULT_DEVICE "/dev/lirc0"

int fd;

int sendIoctl(const char* message, unsigned int cmd, int arg)
{
	printf("%s", message);
	int result = ioctl(fd, cmd, &arg);
	if(result == -1)
	{
		printf("failed (%s)\n", strerror(errno));
	}
	else
	{
		printf("succeeded\n");
	}
	return result;
}

void enableLearningDetector(int enable)
{
	if(enable)
	{
		sendIoctl("Trying to enable learning detector...", LIRC_SET_WIDEBAND_RECEIVER, 1);
	}
	else
	{
		sendIoctl("Trying to disable learning detector...", LIRC_SET_WIDEBAND_RECEIVER, 0);
	}
}

void handleSigInt(int sig)
{
	printf("\nOptions\n");
	printf("e\tEnable learning detector\n");
	printf("d\tDisable learning detector\n");
	printf("c\tCancel prompt\n");
	printf("x\tExit\n");
	printf("> ");
	
	char cmd;
	do
	{
		scanf("%c", &cmd);
	}
	while(cmd == '\n'); // ignore lf from entering previous command
	
	switch(cmd)
	{
		case 'e':
			enableLearningDetector(1);
			break;
		case 'd':
			enableLearningDetector(0);
			break;
		case 'x':
			close(fd);
			exit(0);
			break;
		case 'c':
			break;
		default:
			printf("Unrecognised command %c, ignoring\n", cmd);
			break;
	}
	printf("\n");
}

void showUsage()
{
	printf("irinput [options]\n");
	printf("\n");
	printf("options\n");
	printf("-d device    specify device to use (default %s)\n", DEFAULT_DEVICE);
	printf("-h           print this message and exit\n");
	printf("\n");
}

int main(int argc, char** argv)
{
	const char* device = DEFAULT_DEVICE;
		
	int i = 1;
	for(; i < argc; ++i)
	{
		if(!strcmp(argv[i], "-h"))
		{
			showUsage();
			return 0;
		}
		else if(!strcmp(argv[i], "-d"))
		{
			++i;
			if(i < argc)
			{
				device = argv[i];
			}
			else
			{
				fprintf(stderr, "-d must precede device name\n\n");
				showUsage();
				return 0;
			}
		}
		else
		{
			fprintf(stderr, "Unrecognised option %s\n\n", argv[i]);
			showUsage();
			return 0;
		}
	}
	
	fd = open(device, O_RDWR);
	if(fd == -1)
	{
		fprintf(stderr, "Failed to open %s (%s)\n", device, strerror(errno));
		return -1;
	}
	signal(SIGINT, handleSigInt);
	
	int buffer[BUFFER_SIZE];
	int size = BUFFER_SIZE * sizeof(int);
	int readBytes, readSymbols;
	while(1)
	{
		readBytes = read(fd, (char*)&buffer, size);
		if(readBytes == -1) return -1;
		readSymbols = readBytes / sizeof(int);
		for(i = 0; i < readSymbols; ++i)
		{
			switch(buffer[i] & LIRC_MODE2_MASK)
			{
				case LIRC_MODE2_SPACE:
					printf("space ");
					break;
				case LIRC_MODE2_PULSE:
					printf("pulse ");
					break;
				case LIRC_MODE2_FREQUENCY:
					printf("frequency ");
					break;
				case LIRC_MODE2_TIMEOUT:
					printf("timeout ");
					break;
			}
			printf("%i\n", buffer[i] & LIRC_VALUE_MASK);
		}
	}
	
	// never get to here
	return 0;
}
