#include <fstream>
#include <iostream>

#include "signal-file.h"
#include "double-signal.h"

using namespace std;

bool SignalFile::loadFile(const string& filePath)
{
	ifstream file(filePath.c_str());
	if(!file.good())
	{
		cerr << "Failed to open " << filePath << endl;
		return false;
	}
	string word;
	file >> word;
	if(word != "DEVICE")
	{
		cerr << "Expected to read DEVICE, read " << word << endl;
		return false;
	}
	file >> word;
	cout << "Loading signals for device " << word << endl;
	while(file.good())
	{
		string signalType;
		file >> signalType >> word;
		if(!signalType.empty())
		{
			Signal* signal;
			if(signalType == "MODSIG")
			{
				signal = new ModulatedSignal();
			}
			else if(signalType == "DMODSIG")
			{
				signal = new DoubleSignal();
			}
			else
			{
				cerr << "Signal type " << signalType << " not recognised (signal " << word << ")" << endl;
				return false;
			}
			
			signal->name = word;
			if(signal->fromFile(file))
			{
				signalList[word] = signal;
			}
			else
			{
				cerr << "Loading signal " << word << " failed" << endl;
				// don't return, other signals might be ok
			}
		}
	}
	return !signalList.empty();
}

Signal* SignalFile::getSignal(const std::string& signalName)
{
	map<string, Signal*>::iterator it = signalList.find(signalName);
	return ((it == signalList.end()) ? NULL : it->second);
}

